import os
import sys
import termios
import tty
from contextlib import contextmanager

INTERRUPT = 3
UP_KEY = 65
DOWN_KEY = 66
RIGHT_KEY = 67
LEFT_KEY = 68

NEXT_KEYS = (RIGHT_KEY, DOWN_KEY)
PREV_KEYS = (LEFT_KEY, UP_KEY)


@contextmanager
def raw_stdin():
    """
    This context manager makes sure to put stdin into raw mode before reading
    any data. After the context is done it resets to previous settings.
    """
    try:
        fd = sys.stdin.fileno()
        settings = termios.tcgetattr(fd)
        tty.setraw(fd)
        yield
    finally:
        termios.tcsetattr(fd, termios.TCSADRAIN, settings)


class Presentation:

    def __init__(self):
        self._slides = []
        self._current_slide = None
        self._next_slide = None
        self._previous_slide = None

    def __call__(self):
        self.next()

    def __repr__(self):
        return (f'<Presentation: current_slide: {self._current_slide}, '
                f'next_slide: {self._next_slide}, '
                f'previous_slide: {self._previous_slide}')

    def __clear(self):
        os.system('clear')

    def add_slide(self, func):
        self._slides.append(func)

    def current(self):
        if self._current_slide:
            self.__clear()
            self._current_slide()

    def next(self):
        self.__clear()

        if not self._current_slide:
            self._next_slide = self._slides[0]

        if not self._next_slide:
            return self._current_slide()

        # Defines current slide
        self._current_slide = self._next_slide
        current_slide_pointer = self._slides.index(self._current_slide)

        # Defines previous slide
        if current_slide_pointer > 0:
            self._previous_slide = self._slides[current_slide_pointer - 1]

        # Defines next slide
        # If it's last slide
        if current_slide_pointer == len(self._slides):
            return self._current_slide()

        # If it's one before the last
        if current_slide_pointer == len(self._slides) - 1:
            self._next_slide = None
        else:
            self._next_slide = self._slides[current_slide_pointer + 1]

        self._current_slide()

    def previous(self):
        if self._previous_slide:
            self.__clear()
            self._next_slide = self._previous_slide
            return self.next()

    def _read_input(self):
        with raw_stdin():
            char = sys.stdin.read(1)

        return ord(char)

    def present(self):
        self.next()

        while True:
            key = self._read_input()

            if key in PREV_KEYS:
                self.previous()
            elif key in NEXT_KEYS:
                self.next()
            elif key == INTERRUPT:
                return



class Template:
    def default(self, title, text, height=20, width=80):
        top_symbol = '-'
        bottom_symbol = '-'
        line_symbol = '#'
        blank_space = ' '
        title_line_space = 2
        line_border_size = 4  # because '#' is 'larger' than '-'
        title_size = len(title)

        # Prints top line
        print(top_symbol * width)
        height -= 1

        # Prints blank lines before title
        for _ in range(title_line_space):
            print(line_symbol, blank_space * (width - line_border_size), line_symbol)
            height -= 1

        # Prints title
        print(
            line_symbol,
            blank_space * line_border_size,
            title,
            blank_space * (width - line_border_size - title_size - 6),
            line_symbol,
        )
        height -= 2

        # Prints text
        text_lines = text.split('\n')
        for line in text_lines:
            line_size = len(line)
            print(
                line_symbol,
                blank_space * line_border_size,
                line,
                blank_space * (width - line_border_size - line_size - 6),
                line_symbol,
            )
            height -= 1

        # Prints last blank lines
        for _ in range(height - 1):
            print(line_symbol, blank_space * (width - line_border_size), line_symbol)

        # Prints bottom line
        print(bottom_symbol * width)



import os
import re
from setuptools import setup


def get_version(package):
    init_py = open(os.path.join(package, '__init__.py')).read()
    return re.search("__version__ = ['\"]([^'\"]+)['\"]", init_py).group(1)


def get_long_description(long_description_file):
    with open(long_description_file, encoding='utf-8') as f:
        long_description = f.read()

    return long_description


version = get_version('slydes')


setup(
    name='slydes',
    version=version,
    url='https://github.com/jonatasbaldin/slydes',
    license='MIT',
    description='Why not show your presentations with Python?',
    long_description=get_long_description('README.md'),
    long_description_content_type='text/markdown',
    packages=['slydes'],
    author='Jonatas Baldin',
    author_email='jonatas.baldin@gmail.com',
    install_requires=[],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Topic :: Multimedia :: Graphics :: Presentation',
    ],
)
